<?php
/**
 * 2007-2017 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2017 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!class_exists('MACHelpers')) {
    include_once _PS_MODULE_DIR_ . 'esto/classes/MACHelpers.php';
}

class EstoAPI
{
    private $apiURL;
    private $esto_shop_id = '';
    private $esto_secret_key = '';
    private $api_country = '';

    public function __construct($apiURL, $esto_shop_id=null, $esto_secret_key=null, $api_country=null)
    {
        //$this->apiURL = 'https://api.esto.ee/v2/';
        $this->apiURL = $apiURL;
        if (!empty($esto_shop_id) && !empty($esto_secret_key) && !empty($api_country)) {
            $this->esto_shop_id = $esto_shop_id;
            $this->esto_secret_key = $esto_secret_key;
            $this->api_country = $api_country;
        } else {
            $this->setDefaultCountryApiDetails();
        }

    }

    public function getPurchaseRedirect($data)
    {
        $service = 'purchase';
        $method = 'POST';

        $response = $this->makeCall($service, $data, $method);

        return $response->purchase_url;
    }

    public function getMonthlyPayment($data)
    {
        $service = 'calculate/payment';
        $method = 'POST';

        $response = $this->makeCall($service, $data, $method);

        if (is_null($response) || !isset($response->monthly_payment)) {
            return false;
        }

        return $response->monthly_payment;
    }

    private function makeCall($service, $data = [], $method = 'GET')
    {
        $url = $this->apiURL . $service;

        if($method == 'GET') {
            $url .= '?' . http_build_query($data);
        }

        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);

        // Basic auth
        $user = $this->esto_shop_id;
        $password = $this->esto_secret_key;


        curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
        curl_setopt($ch, CURLOPT_USERPWD, $user . ':' . $password);

        $headers = array(
            'Accept: application/json',
            'Content-Type: application/json',
            //'Authorization: Basic '.base64_encode( $user.':'. $password )
        );
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $data = json_encode($data);

        switch ($method) {
            case 'GET':
                break;

            case 'POST':
                curl_setopt($ch, CURLOPT_POST, true);
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;

            case 'PUT':
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'PUT');
                curl_setopt($ch, CURLOPT_POSTFIELDS, $data);
                break;

            case 'DELETE':
                curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
                break;
        }

        $response = curl_exec($ch);
        $response = json_decode($response);
        if(isset($response->data)) {
            /*$json = htmlspecialchars_decode($response->data);
            $mac = $response->mac;
            if( ! MACHelpers::verifyMessage($json, $mac, $api_country)) {
                die('Illegal access');
            }*/

            return json_decode($response->data);
        } else {
            return $response;
        }
    }


    public function getAvailablePayments($data = null){

        $service = isset($data['service'])?$data['service']:'purchase';


        $url = $this->apiURL . $service;


        $banks = array();

        foreach(esto::API_COUNTRIES as $k=>$v) {

            $country_code = $v;

            $curl = curl_init();

            $connectionMode = Configuration::get('ESTO_CONNECTION_MODE');

            if($connectionMode == "test"){
                $connectionMode = true;
            } else{
                $connectionMode = false;
            }


            curl_setopt($curl, CURLOPT_URL, $url . '/payment-methods?country_code=' . $country_code . '&test_mode=' . $connectionMode);

            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);

            // Basic auth
            $user = $this->esto_shop_id;
            $password = $this->esto_secret_key;

            curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            curl_setopt($curl, CURLOPT_USERPWD, $user . ':' . $password);

            $headers = array(
                'Accept: application/json',
                'Content-Type: application/json',
                //'Authorization: Basic '.base64_encode( $user.':'. $password )
            );
            curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);


            curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
            $auth = curl_exec($curl);
            curl_close($curl);

            $return = json_decode($auth,true);


            // delete items which have is_test = true for live mode
            $return_array = array();
            foreach($return as $k_return => $value_return){

                if(stripos($value_return['key'],"ESTO") !== false){
                    continue;
                }

                if(!$connectionMode){
                    if($value_return['is_test']){
                        continue;
                    }
                }
                $return_array[$k_return] = $value_return;
            }
            // delete items which have is_test = true for live mode

            $banks[$v] = $return_array;

        }

        return $banks;
    }


    protected function setDefaultCountryApiDetails()
    {
        $api_details = Esto::getDefaultCountryApiData();
        if (empty($api_details)) {
            return;
        }
        $this->apiURL = $api_details['url'];
        $this->esto_shop_id = $api_details['user'];
        $this->esto_secret_key = $api_details['pass'];
        $this->api_country = $api_details['country_iso'];
    }

}