<?php
/**
 * 2007-2017 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2017 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!class_exists('EstoAPI')) {
    include_once _PS_MODULE_DIR_ . 'esto/classes/EstoAPI.php';
}

class EstoPaymentModuleFrontController extends ModuleFrontController
{
    public $name = 'esto';

    public function postProcess()
    {
        $module = $this->module;

        if (!Tools::getIsset('payment-option') ||
            !in_array(Tools::getValue('payment-option'), ['ESTO', 'ESTO_X', 'ESTO_PAY_LATER', 'ESTO_PAY']) ||
            ('ESTO_X' === Tools::getValue('payment-option') && $this->context->cart->getOrderTotal() < $module::ESTO_X_MIN_AMOUNT)
        ) {
            Tools::redirect($this->context->link->getPageLink('order'));
        }

        $estoPayment = Tools::getValue('payment-option');
        $cart = $this->context->cart;

        $totalAmount = $cart->getOrderTotal();

        $customer = new Customer($cart->id_customer);
        $billingAddress = new AddressCore($cart->id_address_invoice);
        $connectionMode = Configuration::get('ESTO_CONNECTION_MODE');
        $message = $connectionMode == $module::MODE_TEST ? 'TEST MODE' : null;
        $createOrder = Configuration::get('ESTO_CREATE_ORDER');
        $reference = $cart->id;

        if ($createOrder == $module::CREATE_ORDER_BEFORE) {

            $paymentInfo = str_replace('_', ' ', $estoPayment);
            if ('ESTO_PAY' === $estoPayment && !empty(Tools::getValue('esto-bank-name'))){
                $paymentInfo .= $this->module->getBankPaymentSuffix(Tools::getValue('esto-bank-name'), $cart->id);
            }

            $this->module->validateOrder(
                $cart->id,
                Configuration::get($estoPayment . '_OS_PROCESSING'),
                0,
                $paymentInfo,
                $message,
                [],
                null,
                false,
                $customer->secure_key
            );
            $orderId = $this->module->currentOrder;
            $reference = $this->getExportedOrderReferenceById($orderId);
        }

        $api_details = $this->getApiData($billingAddress, $module);

        $returnPayload = ['esto_payment' => $estoPayment, 'api_country' => $api_details['country_iso']];
        if ('ESTO_PAY' === $estoPayment && !empty(Tools::getValue('esto-bank-name'))){
            $returnPayload['esto-bank-name'] = Tools::getValue('esto-bank-name');
        }

        $data = [
            'amount'            => round($totalAmount, 2),
            'reference'         => (string) $reference,
            'return_url'        => $this->context->link->getModuleLink($this->module->name, 'validation',
                $returnPayload, true),
            'notification_url'  => $this->context->link->getModuleLink($this->module->name, 'validation',
                $returnPayload, true),
            'cancel_url'        => $this->context->link->getModuleLink($this->module->name, 'cancel', #idk why was 'cancel'
                [], true),
            'connection_mode'   => $connectionMode,
        ];

        if ($billingAddress) {
            $data['customer'] = [
                'first_name' => $billingAddress->firstname,
                'last_name' => $billingAddress->lastname,
                'email' => $customer->email,
                'phone' => !empty($billingAddress->phone_mobile) ? $billingAddress->phone_mobile : $billingAddress->phone,
                'address' => $billingAddress->address1,
                'city' => $billingAddress->city,
                'post_code' => $billingAddress->postcode,
            ];
        }

        foreach ($data['customer'] as $key => $val) {
            if (is_null($val) || $val === '') {
                unset($data['customer'][$key]);
            }
        }

        $products = $cart->getProducts(true);
        if ($products && count($products)) {
            $items = [];

            foreach ($products as $product) {
                $items[] = [
                    'name' => $product['name'],
                    'unit_price' => round($product['price_wt'], 2),
                    'quantity' => $product['cart_quantity'],
                ];
            }
            $data['items'] = $items;
        }

        switch ($estoPayment) {
            case 'ESTO_X':
                $data['schedule_type'] = 'ESTO_X';
                break;
            case 'ESTO_PAY_LATER':
                $data['schedule_type'] = 'PAY_LATER';
                break;
            case 'ESTO_PAY':
                $data['schedule_type'] = 'ESTO_PAY';
                break;
        }

        if(Tools::getIsset('esto-bank')) {
            $data['payment_method_key'] = Tools::getValue('esto-bank');
        }
        if(Tools::getIsset('esto-type')) {
            $data['payment_provider'] = Tools::getValue('esto-type');
        }

        $api = new EstoAPI($api_details['url'], $api_details['user'], $api_details['pass'], $api_details['country_iso']);

        Tools::redirect($api->getPurchaseRedirect($data));
    }

    protected function getApiData($billingAddress, $module)
    {
        $api_details = null;

        $api_details = $module::getApiDataByAddress($billingAddress);
        if (!empty($api_details)) {
            return $api_details;
        }

        $estoPayment = Tools::getValue('payment-option');
        if ('ESTO_PAY' === $estoPayment) {
            $bankCountryCode = (Tools::getValue('esto-bank-country-code'));
            if (!empty($bankCountryCode)) {
                $api_details = $module::getApiDataByCountryCode($bankCountryCode);
            }
        }
        if (!empty($api_details)) {
            return $api_details;
        }

        $api_details = $module::getDefaultCountryApiData();

        return $api_details;
    }

    protected function getExportedOrderReferenceById($orderId)
    {
        if (!$this->module->useOrderRefAsReference){
            return $orderId;
        }

        $sql = new DbQuery();
        $sql->select('reference');
        $sql->from('orders', 'o');
        $sql->where('o.id_order = '.(int)$orderId);
        $sql->orderBy('id_order DESC');
        return (Db::getInstance()->getValue($sql) ?: $orderId);
    }
}
