<?php
/**
 * 2007-2017 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2017 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
if (!class_exists('MACHelpers')) {
    include_once _PS_MODULE_DIR_ . 'esto/classes/MACHelpers.php';
}

class EstoValidationModuleFrontController extends ModuleFrontController
{
    const STATUS_APPROVED = 'APPROVED';
    const STATUS_REJECTED = 'REJECTED';

    public $name = 'esto';

    public function postProcess()
    {
        # this block prevents multiple simultaneous executions by several return requests received from esto service
        $filename = 'locked';
        $ds = DIRECTORY_SEPARATOR;
        $fullName = _PS_MODULE_DIR_.$ds.$this->module->name.$ds.$filename;
        $file = fopen($fullName, "w");
        flock($file, LOCK_EX);

        $module = $this->module;
        if ($module->active) {
            $data = $_REQUEST;
            $paymentMessage = htmlspecialchars_decode($data['json']);
            $mac = $data['mac'];
            $connectionMode = Configuration::get('ESTO_CONNECTION_MODE');
            $message = $connectionMode == $module::MODE_TEST ? 'TEST MODE' : null;
            $estoPayment = $data['esto_payment'];
            $apiCountry = $data['api_country'];

            if (!MACHelpers::verifyMessage($paymentMessage, $mac, $apiCountry, true)) {
                die('Illegal access');
            }

            $data = json_decode($paymentMessage, true);
            $cart = null;
            $order = null;
            $customer = null;

            if (Configuration::get('ESTO_CREATE_ORDER') == $module::CREATE_ORDER_AFTER) {
                $cart = new Cart((int) $data['reference']);
                $customer = new Customer($cart->id_customer);

                $cartOrderId = Order::getOrderByCartId($cart->id);

                if (Validate::isLoadedObject($cart) &&
                    $cart->OrderExists() == true &&
                    $cartOrderId &&
                    ($customer->secure_key !== false && $customer->secure_key == $cart->secure_key))
                {
                    $order = new Order($cartOrderId);
                } else {
                    try {
                        $paymentInfo = str_replace('_', ' ', $estoPayment);
                        if ('ESTO_PAY' === $estoPayment && !empty(Tools::getValue('esto-bank-name'))){
                            $paymentInfo .= $this->module->getBankPaymentSuffix(Tools::getValue('esto-bank-name'), $cart->id);
                        }

                        $module->validateOrder(
                            $cart->id,
                            Configuration::get($estoPayment . '_OS_PROCESSING'),
                            0,
                            $paymentInfo,
                            $message,
                            [],
                            null,
                            false,
                            $customer->secure_key
                        );
                        $order = new Order($module->currentOrder);
                    } catch (Exception $e) {
                        //todo add logging
                    }
                }

            } else {
                $orderId = $this->getOrderIdByPayloadReference($data['reference']);
                $order = new Order((int) $orderId);
                $cart = new Cart($order->id_cart);
                $customer = new Customer($order->id_customer);
            }

            // Decide on the response, which OrderState should be set on the order
            $newStateName = 'PS_OS_CANCELED';
            if ($data['status'] == self::STATUS_APPROVED) {
                $newStateName = 'PS_OS_PAYMENT';
            }

            // Set OrderState on the order
            $newState = Configuration::get($newStateName);
            if ($order->current_state != $newState) {
                $order->setCurrentState($newState);
            }

            $this->setPaymentsDescription($order);

            Tools::redirect('index.php?controller=order-confirmation&id_cart=' . $cart->id . '&id_module=' . $this->module->id . '&id_order='
                . $order->id . '&key=' . $customer->secure_key);
        }
    }

    public function setPaymentsDescription($order)
    {
        if (!Validate::isLoadedObject($order)){
            return;
        }

        $orderPayments = $order->getOrderPaymentCollection();

        foreach ($orderPayments as $orderPayment){
            if ('ESTO' === $orderPayment->payment_method){
                if ($orderPayment->payment_method !== $order->payment){
                    $orderPayment->payment_method = $order->payment;
                    $orderPayment->save();
                }
            }
        }
    }

    protected function getOrderIdByPayloadReference($reference)
    {
        if (!$this->module->useOrderRefAsReference){
            return $reference; // $reference is order_id
        }

        $reference = pSQl($reference);
        $sql = new DbQuery();
        $sql->select('id_order');
        $sql->from('orders', 'o');
        $sql->where('o.reference = \''.pSQL($reference).'\'');
        $sql->orderBy('id_order DESC');
        return (Db::getInstance()->getValue($sql) ?: $reference);
    }
}
